if (typeof Object.create !== 'function') {
    Object.create = function (o) {
        var F = function (o) {
        };
        F.prototype = o;
        return new F();
    };
}

Function.prototype.method = function (name, func) {
    if (!this.prototype[name]) {
        this.prototype[name] = func;
        return this;
    }
};

// String functions
//jQuery.string(String.prototype);

jQuery.fn.exists = function (f) {
    var e = jQuery(this);
    if (e.length > 0) {
        if (f) {
            f.apply(e);
        }
        return true;
    }
    else {
        return false;
    }
};

/**
 * Main IOP namespace and common methods.
 */
var IOP = function () {

    /**
     * Parses a URL string into its component parts.
     *
     * @params url      the URL to be parsed
     * @returns         an object of the URL components
     */
    var parseURL = function (url) {
        url = url || window.location;
        var a = document.createElement('a');
        a.href = url;
        return {
            source:url,
            protocol:a.protocol.replace(':', ''),
            host:a.hostname,
            port:a.port,
            query:a.search,
            params:(function () {
                var ret = {},
                    seg = a.search.replace(/^\?/, '').split('&'),
                    len = seg.length, i = 0, s;
                for (; i < len; i++) {
                    if (!seg[i]) {
                        continue;
                    }
                    s = seg[i].split('=');
                    ret[s[0]] = s[1];
                }
                return ret;
            })(),
            file:(a.pathname.match(/\/([^\/?#]+)$/i) || [, ''])[1],
            hash:a.hash.replace('#', ''),
            path:a.pathname.replace(/^([^\/])/, '/$1'),
            relative:(a.href.match(/tps?:\/\/[^\/]+(.+)/) || [, ''])[1],
            segments:a.pathname.replace(/^\//, '').split('/')
        };
    };

    /**
     * Builds a URL from the results produced by the parseURL function.
     * NOTE: It only uses some of the parts of the URL object from the parseURL.
     */
    var buildURL = function (parts, otherParts) {
        var url = jQuery.extend(parts, otherParts);
        return url.protocol + "://" + url.host +
            ((url.port !== "0" && url.port.length > 0) ? ":" + url.port : "") +
            url.path +
            (url.query.length > 0 ? (url.path.endsWith("\?") || url.query.startsWith("\?") ? "" : "?") + url.query : "")
    };

    var notify = function (title, msg) {
        jQuery.pnotify({pnotify_title:title, pnotify_text:msg});
    };

    return jQuery.extend(IOP, {  // make sure IOP is not overridden
        parseURL:parseURL,
        buildURL:buildURL,
        notify:notify
    });

}();


/**
 * Common AJAX methods.
 */
IOP.ajax = function () {

    var waitShow = function (node) {
        if (node === null) {
            return function () {
            };
        }
        else {
            var p = jQuery(node).offset();
            var h = jQuery(node).height();
            var w = jQuery(node).width();
            jQuery(node).css({'zoom':1, 'filter':"alpha(opacity=50)", "-moz-opacity":"0.5", "-khtml-opacity":"0.5", "opacity":"0.5"});
            var waitNode = jQuery("<div style='position: absolute; height: " + h + "px; width: " + w + "px; top: " + p.top + "px; left: " + p.left + "px; z-index: 1000;'><img src='/img/ajax-loader-small-circle.gif' width='16px' height='16px' style='position: absolute; top: " + (Math.floor(h / 2) - 8) + "px; left: " + (Math.floor(w / 2) - 8) + "px;'/></div>");
            waitNode.appendTo("body");
            return function () {
                waitNode.remove();
                jQuery(node).css({'filter':"alpha(opacity=100)", "-moz-opacity":"1", "-khtml-opacity":"1", "opacity":"1"});
            };
        }
        ;
    };

    /**
     * Ajax GET request with wait symbol and default alert box on error
     *
     */
    var requestGet = function (userOptions) {
        var options = jQuery.extend({
            location:"", // url
            params:{}, // parameters
            cache:false,
            element:null, // element to overlay wait symbol
            errorMsg:null, // alert this error message if failure (null ignored)
            success:function () {
            }, // function to call on success
            unique:true, // make URL unique (avoids caching)
            dataType:"json"            // type of reply from server
        }, userOptions);

        var unique = unique || true;
        if (unique) {
            options.params._time = new Date().valueOf();
        }
        var waiter = waitShow(options.element);
        jQuery.ajax({
            type:"GET",
            url:options.location,
            data:options.params,
            cache:options.cache,
            success:function (data, textStatus) {
                options.success(data);
                waiter();
            },
            error:function (request, textStatus, errorThrown) {
                waiter();
                if (options.errorMsg !== null) {
                    alert(options.errorMsg);
                }
            },
            dataType:options.dataType
        });
    };

    var requestPost = function (userOptions) {
        var options = jQuery.extend({
            location:"", // url
            params:{}, // javascript object
            type:"POST",
            dataType:"text",
            contentType:"application/x-www-form-urlencoded",
            dataConvertor:function (d) {
                return d;
            }, // function to convert data
            element:null, // element to overlay wait symbol
            success:function () {
            }, // function to call on success
            errorMsg:null                // alert error message on failure
        }, userOptions);

        var waiter = waitShow(options.element);
        jQuery.ajax({url:options.location,
            type:options.type,
            dataType:options.dataType,
            contentType:options.contentType,
            data:options.dataConvertor(options.params),
            success:function (data) {
                options.success(data);
                waiter();
            },
            error:function (request, textStatus, errorThrown) {
                waiter();
                if (options.errorMsg !== null) {
                    alert(options.errorMsg);
                }
            }});
    };
    
    /**
     * Ajax GET request with wait symbol and default alert box on error
     *
     */
    var requestGetNoCache = function (userOptions) {
        var options = jQuery.extend({
            location:"", // url
            params:{}, // parameters
            element:null, // element to overlay wait symbol
            errorMsg:null, // alert this error message if failure (null ignored)
            success:function () {
            }, // function to call on success
            unique:false, 
            dataType:"json"            // type of reply from server
        }, userOptions);

        var waiter = waitShow(options.element);
        jQuery.ajax({
            type:"GET",
            url:options.location,
            data:options.params,
            success:function (data, textStatus) {
                options.success(data);
                waiter();
            },
            error:function (request, textStatus, errorThrown) {
                waiter();
                if (options.errorMsg !== null) {
                    alert(options.errorMsg);
                }
            },
            dataType:options.dataType
        });
    };

    var requestJsonPost = function (userOptions) {
        return requestPost(jQuery.extend({
            type:"POST",
            dataType:"json",
            contentType:"application/json; charset=utf-8",
            dataConvertor:function (d) {
                return jQuery.toJSON(d);
            }
        }, userOptions));
    };
    
    var requestJsonGet = function (userOptions) {
        return requestGetNoCache(jQuery.extend({
            type:"GET",
            dataType:"json",
            contentType:"application/json; charset=utf-8",
            dataConvertor:function (d) {
                return jQuery.toJSON(d);
            }
        }, userOptions));
    };

    return {
        get:requestGet,
        post:requestPost,
        postJson:requestJsonPost,
        getJson:requestJsonGet
    };

}();

/*
 * ASS Code
/* Object LinkingTool Tip*/
$(window).load(function () {
    $("a.cite.object-linking").click(function () {
        $(".tooltip").hide();
        var objectId = $(this).text();
        var href = $(this).attr('href');
        var objId = "objId=";
        var indexObjId = href.indexOf(objId) + objId.length;
        var indexAmpersand = href.indexOf("&",indexObjId);
        var objectValue = href.substring(indexObjId,indexAmpersand);
        IOP.ajax.postJson({
            location:"relatedobj",
            params:{method:'getObjectLinksByID', params:[objectValue]},
            errorMsg:null,
            success:function (data) {
                var objectLinkText = objectId;
                var simbadLinkText = data.facilityarray.simtextkey;
                var simbadURL = data.facilityarray.simurlkey;
                var nedLinkText = data.facilityarray.nedtextkey;
                var nedURL = data.facilityarray.nedurlkey;
                $(".tooltip").show();
                var objectLinkDiv = "<div class='closebutton'></div><div>" + objectLinkText + "</div><br/>";

                if (simbadLinkText) {
                    objectLinkDiv += "<a href=" + simbadURL + " class='objectLinking' target='_blank'>" + simbadLinkText + "</a>";
                }

                if (nedLinkText) {
                    objectLinkDiv += "<a href=" + nedURL + " class='objectLinking'  target='_blank'>" + nedLinkText + "</a>";
                }

                $(".tooltip").html(objectLinkDiv)
            }
        });
    });
})

$(document).ready(function () {
    $(".figureSetLink, .extendedFigureLink").each(function (index, el) {
        $(el).attr("href", $(el).attr("href").replace("&javascript=false", ""))
    })
});


$(window).load(function () {
    handleDisabledJavaScript();
    setUpFigureSetImg();
    adjustFigureCaption();
    sizing();
});

function handleDisabledJavaScript() {
    jQuery("#imageBoxes").hide();
    jQuery(".bigImage.enabledJavaScript").show();
}

function setUpFigureSetImg() {
    if ($(".fullFigureSetImg ul li img").size() > 0) {
        $(".fullFigureSetImg ul li img").bind('click', function (event) {
            changeImg(event.target);
        });
        $(".fullFigureSetImg ul li .wrapPara img").unbind('click');
        changeImg($(".fullFigureSetImg ul li img")[0]);
    }
}

function adjustFigureCaption() {
    $(".greyContainer > .scroll.fullFigureSetImg div > ul > li").each(function (index, element) {
        var imageWidth = $(element).find("img").width();
        var captionWidth = 0;

        $(element).find(".figureSet-name > p").each(function (i, e) {
            captionWidth = captionWidth + $(e).width();
        });

        if (captionWidth > imageWidth) {
            $(element).find(".figureSet-name > p").css({"display":"block", "margin":0, "padding":0, "white-space":"pre-wrap", "width":imageWidth, "word-wrap": "break-word"});
            $(element).find(".figureSet-name").parent().css({"vertical-align":"top"});
        }
    })
}

function changeImg(img) {
    $(".fullFigureSetImgDark").removeClass("fullFigureSetImgDark");
    $(img).addClass("fullFigureSetImgDark");
    //Replacing the gif src with jpg because we want to show Full image when user clicks on any one of the image in image bar.
    var jpgsrc = (img.src.replace('.gif', '.jpg')).replace('/thumb/', '/Full/');
    var gifsrc = (img.src.replace('/thumb/', '/Full/'));

    checkImage(jpgsrc);
    changeCaption(img);
}

function checkImage(src) {
    var img = new Image();
    img.src = src;
    img.onload = function () {
        // code to set the src on success

    };
    img.onerror = function () {
        // doesn't exist or error loading
        checkImage(src.replace('.jpg', '.gif'));
    };

    $(".zoomableimg").attr("src", src);// fires off loading of image
}

function changeCaption(img) {
    $("#captionDiv").html($(img).parent().siblings()[1].innerHTML);
}

function openFigureSet() {

}


